#ifndef PIXELPIPELINE_H
#define PIXELPIPELINE_H

#include <pipeline.h>
#include <glm/mat4x4.hpp>

class FrameBuffer;
class Texture;
class Shader;
class PixelMesh;
class MapScene;
class Mesh;

class PixelPipeline : public Pipeline
{
private:
   FrameBuffer *m_mapFBO;
   Texture *m_mapTex;
   Shader *m_texMapShader;
   MapScene *m_map;
   int m_mapWidth;
   int m_mapHeight;
   const FrameBuffer *m_targetFBO;
   Shader *m_renderShader;
   int m_width;
   int m_height;
   glm::vec3 m_camera;

   Mesh *m_toreiller;
   glm::mat4 m_mvp;
   //unsigned int m_vao, m_vbo;
   
public:
    PixelPipeline(MapScene *map);
    ~PixelPipeline();

    void updateChanges();
    void setTargetFBO(FrameBuffer *fbo) {m_targetFBO = fbo;}

    virtual void renderGL(Scene *scene);
    virtual void resizeGL(int w, int h);
    
    void cameraMove(int x, int y) { m_camera.x -= x/(m_camera.z*m_width); m_camera.y += y/(m_camera.z*m_width); }
    void cameraZoom(int nbScrolls);
};

#endif // PIXELPIPELINE_H
