#include "simulation.h"

#include <algorithm>
#include "map.h"
#include "dude.h"
#include "mapscene.h"

Simulation::Simulation(MapScene *_map, std::vector<BehaviorFunction> &_behaviors):
    p_map(_map)
{
    int i=0;
    for(BehaviorFunction &behavior : _behaviors){
        glm::vec3 color;
        color[i%3]=0.5f*(1 + i/3);
        m_teams.push_back(Team(color, color, behavior));
        i++;
        // TODO : display the spawns on the map
    }
}

void Simulation::update()
{
    std::random_shuffle(m_dudes.begin(),m_dudes.end());
    for (auto dude : m_dudes){
        auto action = m_teams[dude->getTeam()].update(*dude); //get action for this dude from behavior function in team
        handleAction(action,dude);
    }

    // for each team, spawn dude if condition met
    for(int i=0; i<m_teams.size(); ++i){
        Team &t = m_teams[i];
        if (t.updateSpawn())
        {
            Dir randDir = Dir(rand()%4);
            Coord spawnPos = p_map->team(i) + Coord(randDir);
            m_dudes.push_back(new Dude(spawnPos,p_map,i));
        }
    }
}

// TODO perform action (cf old_code/main.c -> void handleAction(t_dude* dude) )

void Simulation::handleAction(Action *action, Dude *dude){
    Coord targetPos(dude->getPos() + Coord(action->dir));
    Pixel target = p_map->getPixel(targetPos);
    Dude* targetDude;
    dude->setSuccess(false);
    switch(action->type){
    case Action::MOVE:
        if (target.type != WALL && target.type != ROCK
                && target.type != BEDROCK && target.type != IRON_ORE
                && target.type != TREE && target.type != LIBRARY)
            NULL; // TODO: move action
        break;
    case Action::ATTACK:
        if (target.type == DUDE
                || target.type == SPAWN
                || target.type == WALL
                || target.type == LIBRARY
                || target.type == ROAD)
        {
            dude->setSuccess(true);
            if (target.type == DUDE)
                NULL; // TODO:  add fight between dude and targetDude
            else{
                if(target.type == SPAWN)
                    m_teams[target.data.nbRes].destroySpawn();
                target.type = GRASS;
            }
        }
        break;
    case Action::PICK:
        if(target.type >= FOOD && target.type <= SWORD && dude->getInventory() == -1){
            dude->setInventory(target.type);
            target.data.nbRes--;
            if (target.data.nbRes < 1){
                target.type = GRASS;
                // TODO: change color of target
            }
            dude->setSuccess(true);
        }
        break;
    case Action::PUT:
        dude->setSuccess(true);
        if(dude->getInventory() != -1 && (target.type == GRASS || target.type == MARK || target.type == dude->getInventory())){
            if(target.type == GRASS || target.type == MARK){
                target.type = PixelType(dude->getInventory());
                target.data.nbRes = 1;
            }else
                target.data.nbRes++;
            dude->setInventory(-1);
            // TODO: change pixel to new type
        }else if(target.type == SPAWN && dude->getInventory() == FOOD){
            dude->setInventory(-1);
            m_teams[target.data.nbRes].addFood();
        }else{
        //    printf("put failed : trying to put %d in %d\n", dude->inventory, target.type);
            dude->setSuccess(false);
        }
        break;
    case Action::WORK:
        dude->setSuccess(true);
        switch(target.type){
        case ROCK:
            target.type = STONE;
            target.data.nbRes = 1;
            break;    
        case BERRIES:
            target.type = FOOD;
            target.data.nbRes = 1;
            break;
        case TREE:
            target.type = WOOD;
            target.data.nbRes = 1;
            break;
        case IRON_ORE:
            target.type = IRON;
            target.data.nbRes = 1;
            break;
        case GRASS:
            target.type = MARK;
            break;
        case MARK:
            target.type = GRASS;
            break;
        case WOOD:
            switch(target.data.nbRes){
            case 1:
                target.type = WALL;
                break;
            case 2:
                target.type = LIBRARY;
                //TODO : allocate 128 byte in data ?
                break;
            default:
                dude->setSuccess(true);
                break;
            }
        case STONE:
            if(target.data.nbRes == 1)
                target.type = ROAD;
            else
                dude->setSuccess(false);
            break;
        case IRON:
            if(target.data.nbRes == 1)
                target.type = SWORD;
            else
                dude->setSuccess(false);
            break;
        default:
            dude->setSuccess(false);
            break;
        }
        // TODO : update pixel
        break;
    case Action::WAIT:
        dude->setSuccess(true);
        break;
    case Action::COMMUNICATE:
        switch(target.type){
        case DUDE:
            action->com_data.flag = (action->dir+2)%4;
            // TODO : find a way to get targetDude
            //targetDude =
            if(targetDude->getCom().data == NULL){
                targetDude->receiveComData(action->com_data);
                dude->setSuccess(true);
            }else
                dude->setSuccess(false);
            break;
        case LIBRARY:
            if(action->com_data.flag & Com::READ)
            {
                if(dude->getCom().data == NULL)
                {
                    action->com_data.flag = action->dir;
                    dude->receiveComData(action->com_data);
                    // TODO: understand what the fuck this line is doing
                    //memcpy(dude->getCom() + sizeof(int), target.data.knowledge + (action->com_data.flag | 3), COM_SIZE);
                }
                else
                {
                    dude->setSuccess(false);
                    break;
                }
            }
            else{
                // TODO: understand what the fuck this line is doing bis
                //memcpy(target.data.knowledge + action->com_data.flag, action->com_data.data, COM_SIZE);
                dude->setSuccess(true);
            }
            break;
        default:
            dude->setSuccess(false);
            break;
        }
    }
}
